	module HCOUP_SM
        use SM_const
	use pv
	use Gauge_boson_2p
	implicit none

	contains
!-===============  FUNCTIONS ============================
!##########################################################
!
!    SECTION I Tree-level couplings    
!
!##########################################################

! These lambda couplings are defined as the coefficients in Lagrangian. 

      double precision function lam_hhh_sm(mh)
	double precision, intent(in):: mh

	lam_hhh_sm = -mh**2/(2.d0*vev)

	end function
!--------------------------------------------
        double precision function lam_GpGmh_sm(mh)
	double precision, intent(in):: mh

	lam_GpGmh_sm = -mh**2/vev

	 end function 
!--------------------------------------------
      double precision function lam_GGh_sm(mh)
	double precision, intent(in):: mh

	lam_GGh_sm = -mh**2/(2.d0*vev)

	 end function 
!--------------------------------------------
      double precision function lam_hhhh_sm(mh)
	double precision, intent(in):: mh

	lam_hhhh_sm = -mh**2/(8.d0*vev**2)

	 end function 
!--------------------------------------------
      double precision function lam_GpGmhh_sm(mh)
	double precision, intent(in):: mh

	lam_GpGmhh_sm = -mh**2/(2.d0*vev**2)

	 end function 
!--------------------------------------------
      double precision function lam_GGhh_sm(mh)
	double precision, intent(in):: mh

	lam_GGhh_sm = -mh**2/(4.d0*vev**2)

	 end function 
!##########################################################
!
!    SECTION II IPI DIAGRAMS
!
!##########################################################
!====== 1-point functions ========
	complex (kind(0d0)) function Th_fermion(mf)
	double precision, intent(in):: mf

	Th_fermion = -4.d0*mf**2/vev/(16.d0*pi**2)*a0f(mf)

	 end function
!--------------------------------------------------------
	complex (kind(0d0)) function Th_F_SM()

        Th_F_SM = nc*Th_fermion(mt)&
                 +nc*Th_fermion(mb)&
                 +nc*Th_fermion(mc)&
                 +Th_fermion(mtau)

	 end function
!---------------------------------------------
	complex (kind(0d0)) function Th_B_SM()
	double precision x

	Th_B_SM =  1.d0/(16.d0*pi**2)*&
	       (&	
	        -3.d0*lam_hhh_sm(mh)*a0f(mh)& 
		-lam_GpGmh_sm(mh)*a0f(mw)-lam_GGh_sm(mh)*a0f(mz)&
	        +3.d0*g*mW*a0f(mW)+3.d0/2.d0*gz*mZ*a0f(mZ)-2.d0*g*mW**3-gz*mZ**3&
		)

	 end function
!====== 2-point functions ========

!====== Higgs boson two point functions ========
	complex (kind(0d0)) function PiGG_fermion(qsq,mf)
	double precision, intent(in):: qsq,mf

	PiGG_fermion=&
	 -4.d0*mf**2/(16.d0*pi**2*vev**2)*&
	( a0f(mf) -qsq/2.d0*b0f(qsq,mf,mf) )

	 end function
!----------------------------------------------------
	complex (kind(0d0)) function Pihh_fermion(qsq,mf)
	double precision, intent(in):: qsq,mf

	Pihh_fermion=&
	 -4.d0*mf**2/(16.d0*pi**2*vev**2)*&
	( a0f(mf) + (2.d0*mf**2-qsq/2.d0)*b0f(qsq,mf,mf) )

	 end function
!-----------------------------------------------------
	complex (kind(0d0)) function dPihh_fermion(qsq,mf)
	double precision, intent(in):: qsq,mf

	dPihh_fermion=&
	 -4.d0*mf**2/(16.d0*pi**2*vev**2)*&
	( (2.d0*mf**2-qsq/2.d0)*db0f(qsq,mf,mf)-1.d0/2.d0*b0f(qsq,mf,mf) )

	 end function
!=======================================================
	complex (kind(0d0)) function Pihh_F_SM(qsq)
	double precision, intent(in):: qsq

	Pihh_F_SM= nc*Pihh_fermion(qsq,mt)&
                    +nc*Pihh_fermion(qsq,mc)&
                    +nc*Pihh_fermion(qsq,mb)&
                    +Pihh_fermion(qsq,mtau)
	
	 end function
!---------------------------------------------
	complex (kind(0d0)) function Pihh_B_SM(qsq)
	double precision, intent(in):: qsq

	Pihh_B_SM=&
	 1.d0/(16.d0*pi**2)*&
	(&
	  g**2*(3.d0*mw**2-qsq)*b0f(qsq,mW,mW)&
	  +g**2/2.d0*(3.d0)*a0f(mW)&
	  +gz**2/2.d0*(3.d0*mz**2-qsq)*b0f(qsq,mZ,mZ)&
	  +gz**2/4.d0*(3.d0)*a0f(mZ)&
	  -(3.d0/2.d0)*(2.d0*g**2*mW**2+gz**2*mZ**2)&
	)&
!
	+1.d0/(16.d0*pi**2)*&
	( -12.d0*lam_hhhh_sm(mh)*a0f(mh)&
	 -2.d0*lam_GpGmhh_sm(mh)*a0f(mw)-2.d0*lam_GGhh_sm(mh)*a0f(mz)&
	 +lam_GpGmh_sm(mh)**2*b0f(qsq,mw,mw)&
	 +2.d0*lam_GGh_sm(mh)**2*b0f(qsq,mz,mz)+18.d0*lam_hhh_sm(mh)**2*b0f(qsq,mh,mh)&
	 	)
	
	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function dPihh_F_SM(qsq)
	double precision, intent(in):: qsq

	dPihh_F_SM = nc*dPihh_fermion(qsq,mt)&
                    +nc*dPihh_fermion(qsq,mc)&
                    +nc*dPihh_fermion(qsq,mb)&
                    +dPihh_fermion(qsq,mtau)
	
	 end function
!---------------------------------------------
	complex (kind(0d0)) function dPihh_B_SM(qsq)
	double precision, intent(in):: qsq

	dPihh_B_SM=&
	 1.d0/(16.d0*pi**2)*&
	(&
	  g**2*(3.d0*mw**2-qsq)*db0f(qsq,mW,mW)&
	  +g**2*(-1.d0)*b0f(qsq,mW,mW)&
	  +gz**2/2.d0*(3.d0*mz**2-qsq)*db0f(qsq,mZ,mZ)&
	  +gz**2/2.d0*(-1.d0)*b0f(qsq,mZ,mZ)&
	)&
!
	+1.d0/(16.d0*pi**2)*&
	(&
	 +lam_GpGmh_SM(mh)**2*db0f(qsq,mw,mw)&
	 +2.d0*lam_GGh_SM(mh)**2*db0f(qsq,mz,mz)&
	 +18.d0*lam_hhh_SM(mh)**2*db0f(qsq,mh,mh)&
	)
	
	 end function
!======= fermion 2-point functions ========================
	complex (kind(0d0)) function Piff_S_SM(qsq,mf)
	double precision, intent(in):: qsq,mf
	double precision:: vf,af,qf,mfp
        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	Piff_S_SM = -1.d0/(16.d0*pi**2)*(&
	2.d0*(g*sw)**2*Qf**2*(2.d0*b0f(qsq,mf,mphoton)-1.d0)&
	+2.d0*gz**2*(vf**2-af**2)*(2.d0*b0f(qsq,mf,mZ)-1.d0)&
	+mf**2/vev**2*(b0f(qsq,mf,mz)-b0f(qsq,mf,mh)&
	+2.d0*mfp**2/mf**2*b0f(qsq,mfp,mw)&	
	            )&
	)
	
	 end function
!---------------------------------------------
	complex (kind(0d0)) function dPiff_S_SM(qsq,mf)
	double precision, intent(in):: qsq,mf
	double precision:: vf,af,qf,mfp
        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	dPiff_S_SM = -1.d0/(16.d0*pi**2)*(&
	2.d0*(g*sw)**2*Qf**2*(2.d0*db0f(qsq,mf,mphoton))&
	+2.d0*gz**2*(vf**2-af**2)*(2.d0*db0f(qsq,mf,mZ))&
	+mf**2/vev**2*(db0f(qsq,mf,mz)-db0f(qsq,mf,mh)&
	+2.d0*mfp**2/mf**2*db0f(qsq,mfp,mw)&	
	            )&
	)
	
	 end function
!---------------------------------------------
	complex (kind(0d0)) function Piff_V_SM(qsq,mf)
	double precision, intent(in):: qsq,mf
	double precision:: vf,af,qf,mfp
        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	Piff_V_SM = -1.d0/(16.d0*pi**2)*(&
	(g*sw)**2*Qf**2*(2.d0*b1f(qsq,mf,mphoton)+1.d0)&
	+gz**2*(vf**2+af**2)*(2.d0*b1f(qsq,mf,mZ)+1.d0)&
	+g**2/4.d0*(2.d0*b1f(qsq,mfp,mW)+1.d0)&
	+mf**2/vev**2*(b1f(qsq,mf,mh)+b1f(qsq,mf,mz)&
	+(mfp**2+mf**2)/mf**2*b1f(qsq,mfp,mw)&	
	            )&
	)
	
	 end function
!---------------------------------------------
	complex (kind(0d0)) function dPiff_V_SM(qsq,mf)
	double precision, intent(in):: qsq,mf
	double precision:: vf,af,qf,mfp
        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	dPiff_V_SM = -1.d0/(16.d0*pi**2)*(&
	(g*sw)**2*Qf**2*(2.d0*db1f(qsq,mf,mphoton))&
	+gz**2*(vf**2+af**2)*(2.d0*db1f(qsq,mf,mZ))&
	+g**2/4.d0*(2.d0*db1f(qsq,mfp,mW))&
	+mf**2/vev**2*(db1f(qsq,mf,mh)+db1f(qsq,mf,mz)&
	+(mfp**2+mf**2)/mf**2*db1f(qsq,mfp,mw)&	
	            )&
	)
	
	 end function
!====== Gauge boson two point functions ========
	complex (kind(0d0)) function PiWW_B_SM(qsq)
	double precision, intent(in):: qsq
	
	PiWW_B_SM=PiWW_B_WOH(qsq)+g**2/(16.d0*pi**2)*( mw**2*b0f(qsq,mh,mW)+1.d0/4.d0*b5f(qsq,mh,mw) )

 	end function
!---------------------------------------------
	complex (kind(0d0)) function PiZZ_B_SM(qsq)
	double precision, intent(in):: qsq
	
	PiZZ_B_SM=PiZZ_B_WOH(qsq)+ g**2/cw**2/(16.d0*pi**2)*( mZ**2*b0f(qsq,mh,mZ)+1.d0/4.d0*b5f(qsq,mh,mZ) )

 	end function
!---------------------------------------------
	complex (kind(0d0)) function PiGG_B_SM(qsq)
	double precision, intent(in):: qsq
	
	PiGG_B_SM = PiGG_B_WOH(qsq)

 	end function
!---------------------------------------------
	complex (kind(0d0)) function PiZG_B_SM(qsq)
	double precision, intent(in):: qsq
	
	PiZG_B_SM = PiZG_B_WOH(qsq)

 	end function
!---------------------------------------------
	complex (kind(0d0)) function dPiGG_B_SM(qsq)
	double precision, intent(in):: qsq
	
	dPiGG_B_SM = dPiGG_B_WOH(qsq)

 	end function
!============== 3-point functions  ================================
        complex (kind(0d0)) function Gam_hhh_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	double precision:: p12
	p12 = 1.d0/2.d0*(qsq-p1sq-p2sq)

	Gam_hhh_F_SM = &
      -8.d0*mt**4*3.d0/(16.d0*pi**2)/vev**3*&
      (&
       b0f(p1sq,mt,mt)+b0f(p2sq,mt,mt)+b0f(qsq,mt,mt)&
      +(&
         ( 4.d0*mt**2-qsq+(qsq-p1sq-p2sq)/2.d0 )*c0f(p1sq,p2sq,qsq,mt,mt,mt)&
        )&
      )&
! 
      -8.d0*mc**4*3.d0/(16.d0*pi**2)/vev**3*&
      (&
       b0f(p1sq,mc,mc)+b0f(p2sq,mc,mc)+b0f(qsq,mc,mc)&
      +(&
         ( 4.d0*mc**2-qsq+(qsq-p1sq-p2sq)/2.d0 )*c0f(p1sq,p2sq,qsq,mc,mc,mc)&
        )&
      )&
!
      -8.d0*mb**4*3.d0/(16.d0*pi**2)/vev**3*&
      (&
       b0f(p1sq,mb,mb)+b0f(p2sq,mb,mb)+b0f(qsq,mb,mb)&
      +(&
         ( 4.d0*mb**2-qsq+(qsq-p1sq-p2sq)/2.d0 )*c0f(p1sq,p2sq,qsq,mb,mb,mb)&
      )&
      )&
! 
      -8.d0*mtau**4/(16.d0*pi**2)/vev**3*&
      (&
       b0f(p1sq,mtau,mtau)+b0f(p2sq,mtau,mtau)+b0f(qsq,mtau,mtau)&
      +(&
         ( 4.d0*mtau**2-qsq+(qsq-p1sq-p2sq)/2.d0 )*c0f(p1sq,p2sq,qsq,mtau,mtau,mtau)&
      )&
      )
	
	 end function
!---------------------------------------------
	complex (kind(0d0)) function Gam_hhh_B_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam_hhh_B_SM =&
       1.d0/(16.d0*pi**2)*(&
!~~~~~~~~~~~ 3-Gauge ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
       +8.d0*g**3*mw**3*c0f(p1sq,p2sq,qsq,mW,mW,mW)&
       -1.d0/2.d0*g**3*mw**3*c0f(p1sq,p2sq,qsq,mw,mw,mw)&
       +4.d0*gz**3*mz**3*c0f(p1sq,p2sq,qsq,mZ,mZ,mZ)&
       -1.d0/4.d0*gz**3*mz**3*c0f(p1sq,p2sq,qsq,mz,mz,mz)&
!~~~~~~~~~~~ 2-Gauge ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
      -1.d0/2.d0*g**3*mw*Chhh_SVV(p1sq,p2sq,qsq,mw,mW,mW)&
      -1.d0/4.d0*gz**3*mz*Chhh_SVV(p1sq,p2sq,qsq,mz,mZ,mZ)&
!~~~~~~~~~~~ 1-Gauge ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
      +1.d0/2.d0*g**2*lam_GpGmh_sm(mh)*Chhh_VSS(p1sq,p2sq,qsq,mW,mw,mw)&
      +1.d0/2.d0*gz**2*lam_GGh_sm(mh)*Chhh_VSS(p1sq,p2sq,qsq,mZ,mz,mz)&
!~~~~~~~~~~~ 0-Gauge ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
      -2.d0*lam_GpGmh_sm(mh)**3*c0f(p1sq,p2sq,qsq,mw,mw,mw)&
      -8.d0*lam_GGh_sm(mh)**3*c0f(p1sq,p2sq,qsq,mz,mz,mz)&
      -216.d0*lam_hhh_sm(mh)**3*c0f(p1sq,p2sq,qsq,mh,mh,mh)&
!~~~~~~~~~~~ B function ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
      +2.d0*g**3*mw*(b0f(p1sq,mW,mW)+b0f(p2sq,mW,mW)+b0f(qsq,mW,mW))&        !! qsq can be separated into p1sq and p2sq and qsq
      +gz**3*mz*(b0f(p1sq,mZ,mZ)+b0f(p2sq,mZ,mZ)+b0f(qsq,mZ,mZ))&
      +2.d0*lam_GpGmh_sm(mh)*lam_GpGmhh_sm(mh)*&
        (b0f(p1sq,mw,mw)+b0f(p2sq,mw,mw)+b0f(qsq,mw,mw))&
      +4.d0*lam_GGh_sm(mh)*lam_GGhh_sm(mh)*&
        (b0f(p1sq,mz,mz)+b0f(p2sq,mz,mz)+b0f(qsq,mz,mz))&
      +72.d0*lam_hhh_sm(mh)*lam_hhhh_sm(mh)*&
        (b0f(p1sq,mh,mh)+b0f(p2sq,mh,mh)+b0f(qsq,mh,mh))&
     )&
	-1.d0/(16.d0*pi**2)*(3.d0*g**3*mW+3.d0/2.d0*gz**3*mZ)

	 end function
!===================================================================
	complex (kind(0d0)) function Gam_hff_S_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	Gam_hff_S_SM = 1.d0/(16.d0*pi**2)*mf/vev*(&
	-4.d0*gz**2*(vf**2-af**2)*Chff_FVF(p1sq,p2sq,qsq,mf,mz)&
	-4.d0*(g*sw)**2*Qf**2*Chff_FVF(p1sq,p2sq,qsq,mf,mphoton)&
	+mf**2/vev**2*Chff_FSF(p1sq,p2sq,qsq,mf,mh)&
	-mf**2/vev**2*Chff_FSF(p1sq,p2sq,qsq,mf,mz)&
	-2.d0*mfp**2/vev**2*Chff_FSF(p1sq,p2sq,qsq,mfp,mw)&
	-2.d0*gz**4*vev**2*(vf**2-af**2)*c0f(p1sq,p2sq,qsq,mZ,mf,mZ)&
	-6.d0*lam_hhh_sm(mh)*mf**2/vev*c0f(p1sq,p2sq,qsq,mh,mf,mh)&
	+2.d0*lam_GGh_sm(mh)*mf**2/vev*c0f(p1sq,p2sq,qsq,mz,mf,mz)&
	+2.d0*lam_GpGmh_sm(mh)*mfp**2/vev*c0f(p1sq,p2sq,qsq,mw,mfp,mw)&
	-g**2/4.d0*Chff_VFS(p1sq,p2sq,qsq,mW,mfp,mw)&
	-g**2/4.d0*Chff_SFV(p1sq,p2sq,qsq,mw,mfp,mW)&
	-gz**2/8.d0*Chff_VFS(p1sq,p2sq,qsq,mZ,mf,mz)&
	-gz**2/8.d0*Chff_SFV(p1sq,p2sq,qsq,mz,mf,mZ)&
	)

	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_P_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_P_SM = 1.d0/(16.d0*pi**2)*mf/vev*(& 
           -g**2/4.d0*(Chff_VFS(p1sq,p2sq,qsq,mW,mfp,mw)&
                      -Chff_SFV(p1sq,p2sq,qsq,mw,mfp,mW))&
           -2.d0*gz**2*vf*af*(Chff_VFS(p1sq,p2sq,qsq,mZ,mf,mz)&
                      -Chff_SFV(p1sq,p2sq,qsq,mz,mf,mZ))&              
                                       )

	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_V1_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_V1_SM = 1.d0/(16.d0*pi**2)*(& 
             2.d0*gz**2*(vf**2 + af**2)*mf**2/vev*(c0f(p1sq,p2sq,qsq,mf,mZ,mf)&
                                              +2.d0*c11f(p1sq,p2sq,qsq,mf,mZ,mf))&
            +2.d0*(g*sw)**2*Qf**2*mf**2/vev*(c0f(p1sq,p2sq,qsq,mf,mphoton,mf)&
                                      +2.d0*c11f(p1sq,p2sq,qsq,mf,mphoton,mf))&
            +g**2/2.d0*mfp**2/vev*(c0f(p1sq,p2sq,qsq,mfp,mW,mfp)&
                            +2.d0*c11f(p1sq,p2sq,qsq,mfp,mW,mfp))&
            +mf**4/vev**3*(c0f(p1sq,p2sq,qsq,mf,mh,mf)&
                            +2.d0*c11f(p1sq,p2sq,qsq,mf,mh,mf))&
            +mf**4/vev**3*(c0f(p1sq,p2sq,qsq,mf,mz,mf)&
                            +2.d0*c11f(p1sq,p2sq,qsq,mf,mz,mf))&
            +mfp**2/vev**3*(mf**2+mfp**2)*(c0f(p1sq,p2sq,qsq,mfp,mw,mfp)&
                             +2.d0*c11f(p1sq,p2sq,qsq,mfp,mw,mfp))&
            -gz**4*vev*(vf**2 + af**2)*(c0f(p1sq,p2sq,qsq,mZ,mf,mZ)&
                                      +c11f(p1sq,p2sq,qsq,mZ,mf,mZ))&
            -g**4/4.d0*vev*(c0f(p1sq,p2sq,qsq,mW,mfp,mW)&
                           +c11f(p1sq,p2sq,qsq,mW,mfp,mW))&
             -6.d0*lam_hhh_sm(mh)*mf**2/vev**2*(c0f(p1sq,p2sq,qsq,mh,mf,mh)&
                            +c11f(p1sq,p2sq,qsq,mh,mf,mh))&
             -2.d0*lam_GGh_sm(mh)*mf**2/vev**2*(c0f(p1sq,p2sq,qsq,mz,mf,mz)&
                            +c11f(p1sq,p2sq,qsq,mz,mf,mz))&
             -1.d0*lam_GpGmh_sm(mh)*(mf**2+mfp**2)/vev**2*(c0f(p1sq,p2sq,qsq,mw,mfp,mw)&
                            +c11f(p1sq,p2sq,qsq,mw,mfp,mw))&
             -g**2/4.d0*mfp**2/vev*(2.d0*c0f(p1sq,p2sq,qsq,mW,mfp,mw)&
                                        +c11f(p1sq,p2sq,qsq,mw,mfp,mw)&
                                   -c0f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                   +c11f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                   )&
             -gz**2/8.d0*mf**2/vev*(2.d0*c0f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                        +c11f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                   -c0f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                   +c11f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                   )&
                                       )

	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_V2_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_V2_SM = 1.d0/(16.d0*pi**2)*(& 
             2.d0*gz**2*(vf**2 + af**2)*mf**2/vev*(c0f(p1sq,p2sq,qsq,mf,mZ,mf)&
                                              +2.d0*c12f(p1sq,p2sq,qsq,mf,mZ,mf))&
            +2.d0*(g*sw)**2*Qf**2*mf**2/vev*(c0f(p1sq,p2sq,qsq,mf,mphoton,mf)&
                                      +2.d0*c12f(p1sq,p2sq,qsq,mf,mphoton,mf))&
            +g**2/2.d0*mfp**2/vev*(c0f(p1sq,p2sq,qsq,mfp,mW,mfp)&
                            +2.d0*c12f(p1sq,p2sq,qsq,mfp,mW,mfp))&
            +mf**4/vev**3*(c0f(p1sq,p2sq,qsq,mf,mh,mf)&
                            +2.d0*c12f(p1sq,p2sq,qsq,mf,mh,mf))&
            +mf**4/vev**3*(c0f(p1sq,p2sq,qsq,mf,mz,mf)&
                            +2.d0*c12f(p1sq,p2sq,qsq,mf,mz,mf))&
            +mfp**2/vev**3*(mf**2+mfp**2)*(c0f(p1sq,p2sq,qsq,mfp,mw,mfp)&
                            +2.d0*c12f(p1sq,p2sq,qsq,mfp,mw,mfp))&
            -gz**4*vev*(vf**2 + af**2)*(c12f(p1sq,p2sq,qsq,mZ,mf,mZ))&
            -g**4/4.d0*vev*(c12f(p1sq,p2sq,qsq,mW,mfp,mW))&
            -6.d0*lam_hhh_sm(mh)*mf**2/vev**2*(c12f(p1sq,p2sq,qsq,mh,mf,mh))&
            -2.d0*lam_GGh_sm(mh)*mf**2/vev**2*(c12f(p1sq,p2sq,qsq,mz,mf,mz))&
            -1.d0*lam_GpGmh_sm(mh)*(mf**2+mfp**2)/vev**2*(c12f(p1sq,p2sq,qsq,mw,mfp,mw))&
            -g**2/4.d0*mfp**2/vev*(2.d0*c0f(p1sq,p2sq,qsq,mW,mfp,mw)&
                                       +c12f(p1sq,p2sq,qsq,mw,mfp,mw)&
                                  -c0f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                  +c12f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                  )&
            -gz**2/8.d0*mf**2/vev*(2.d0*c0f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                       +c12f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                  -c0f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                  +c12f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                  )&
                                       )

	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_A1_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_A1_SM = 1.d0/(16.d0*pi**2)*(& 
             -4.d0*gz**2*vf*af*mf**2/vev*(c0f(p1sq,p2sq,qsq,mf,mZ,mf)&
                                    +2.d0*c11f(p1sq,p2sq,qsq,mf,mZ,mf))&   
             -g**2/2.d0*mfp**2/vev*(c0f(p1sq,p2sq,qsq,mfp,mW,mfp)&
                                    +2.d0*c11f(p1sq,p2sq,qsq,mfp,mW,mfp))&
              +mfp**2*(mf**2 - mfp**2)/vev**3*(c0f(p1sq,p2sq,qsq,mfp,mw,mfp)&
                                     +2.d0*c11f(p1sq,p2sq,qsq,mfp,mw,mfp))&
              +2.d0*gz**4*vf*af*vev*(c0f(p1sq,p2sq,qsq,mZ,mf,mZ)&
                                    +c11f(p1sq,p2sq,qsq,mZ,mf,mZ))&
              +g**4/4.d0*vev*(c0f(p1sq,p2sq,qsq,mW,mfp,mW)&
                                    +c11f(p1sq,p2sq,qsq,mW,mfp,mW))&
              -lam_GpGmh_sm(mh)*(mf**2-mfp**2)/vev**2*(c0f(p1sq,p2sq,qsq,mw,mfp,mw)&
                                    +c11f(p1sq,p2sq,qsq,mw,mfp,mw))&
             +g**2*mfp**2/(4.d0*vev)*(2.d0*c0f(p1sq,p2sq,qsq,mW,mfp,mw)&
                                      +c11f(p1sq,p2sq,qsq,mW,mfp,mw)&
                                      -c0f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                       +c11f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                      )&
             +2.d0*gz**2*vf*af*mf**2/vev*(2.d0*c0f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                      +c11f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                      -c0f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                       +c11f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                      )&
         )

	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_A2_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_A2_SM = 1.d0/(16.d0*pi**2)*(& 
             -4.d0*gz**2*vf*af*mf**2/vev*(c0f(p1sq,p2sq,qsq,mf,mZ,mf)&
                                    +2.d0*c12f(p1sq,p2sq,qsq,mf,mZ,mf))&   
             -g**2/2.d0*mfp**2/vev*(c0f(p1sq,p2sq,qsq,mfp,mW,mfp)&
                                    +2.d0*c12f(p1sq,p2sq,qsq,mfp,mW,mfp))&
             +mfp**2*(mf**2- mfp**2)/vev**3*(c0f(p1sq,p2sq,qsq,mfp,mw,mfp)&
                                    +2.d0*c12f(p1sq,p2sq,qsq,mfp,mw,mfp))&
             +2.d0*gz**4*vf*af*vev*(c12f(p1sq,p2sq,qsq,mZ,mf,mZ))&
             +g**4/4.d0*vev*(c12f(p1sq,p2sq,qsq,mW,mfp,mW))&
             -lam_GpGmh_sm(mh)*(mf**2-mfp**2)/vev**2*(c12f(p1sq,p2sq,qsq,mw,mfp,mw))&
             +g**2*mfp**2/(4.d0*vev)*(2.d0*c0f(p1sq,p2sq,qsq,mW,mfp,mw)&
                                      +c12f(p1sq,p2sq,qsq,mW,mfp,mw)&
                                      -c0f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                       +c12f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                      )&
             +2.d0*gz**2*vf*af*mf**2/vev*(2.d0*c0f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                      +c12f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                      -c0f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                       +c12f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                      )&
         )

	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_T_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_T_SM = 1.d0/(16.d0*pi**2)*(mf/vev)*(&
            mf**2/vev**2*(c11f(p1sq,p2sq,qsq,mf,mh,mf)-c12f(p1sq,p2sq,qsq,mf,mh,mf))&
           -mf**2/vev**2*(c11f(p1sq,p2sq,qsq,mf,mz,mf)-c12f(p1sq,p2sq,qsq,mf,mz,mf))&
           -2.d0*mfp**2/vev**2*(c11f(p1sq,p2sq,qsq,mfp,mw,mfp)-c12f(p1sq,p2sq,qsq,mfp,mw,mfp))&
           +g**2/4.d0*(2.d0*c0f(p1sq,p2sq,qsq,mW,mfp,mw)&
                      +2.d0*c11f(p1sq,p2sq,qsq,mW,mfp,mw)&
                      -1.d0*c12f(p1sq,p2sq,qsq,mW,mfp,mw)&
                      +1.d0*c0f(p1sq,p2sq,qsq,mw,mfp,mW)&
                      +1.d0*c11f(p1sq,p2sq,qsq,mw,mfp,mW)&
                      -2.d0*c12f(p1sq,p2sq,qsq,mw,mfp,mW) )&
           +gz**2/8.d0*(2.d0*c0f(p1sq,p2sq,qsq,mZ,mf,mz)&
                      +2.d0*c11f(p1sq,p2sq,qsq,mZ,mf,mz)&
                      -1.d0*c12f(p1sq,p2sq,qsq,mZ,mf,mz)&
                      +1.d0*c0f(p1sq,p2sq,qsq,mz,mf,mZ)&
                      +1.d0*c11f(p1sq,p2sq,qsq,mz,mf,mZ)&
                      -2.d0*c12f(p1sq,p2sq,qsq,mz,mf,mZ)&
                      )&
                      )
 
	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_PT_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_PT_SM = 1.d0/(16.d0*pi**2)*(mf/vev)*(&
           +g**2/4.d0*(2.d0*c0f(p1sq,p2sq,qsq,mW,mfp,mw)&
                      +2.d0*c11f(p1sq,p2sq,qsq,mW,mfp,mw)&
                      -1.d0*c12f(p1sq,p2sq,qsq,mW,mfp,mw)&
                      -1.d0*c0f(p1sq,p2sq,qsq,mw,mfp,mW)&
                      -1.d0*c11f(p1sq,p2sq,qsq,mw,mfp,mW)&
                      +2.d0*c12f(p1sq,p2sq,qsq,mw,mfp,mW) )&
           +2.d0*gz**2*af*vf*(2.d0*c0f(p1sq,p2sq,qsq,mZ,mf,mz)&
                      +2.d0*c11f(p1sq,p2sq,qsq,mZ,mf,mz)&
                      -1.d0*c12f(p1sq,p2sq,qsq,mZ,mf,mz)&
                      -1.d0*c0f(p1sq,p2sq,qsq,mz,mf,mZ)&
                      -1.d0*c11f(p1sq,p2sq,qsq,mz,mf,mZ)&
                      +2.d0*c12f(p1sq,p2sq,qsq,mz,mf,mZ)&
                      )&
                      )
 
	 end function
!==============================================================
 	complex (kind(0d0)) function Gam1_hZZ_fermion(p1sq,p2sq,qsq,mf)
 	double precision, intent(in):: p1sq,p2sq,qsq,mf
 	double precision:: vf,af

       if (mf .eq. mt) then 
        vf = vt ; af = at
       elseif (mf .eq. mc) then 
        vf = vt ; af = at
       elseif (mf .eq. mb) then 
        vf = vb ; af = ab
       elseif (mf .eq. mtau) then 
        vf = ve ; af = ae
       endif

	Gam1_hZZ_fermion = &
	 -32.d0*mf**2*mz**2/vev**3*&
	 1.d0/(16.d0*pi**2)*&
	 (&
	 -1.d0/2.d0*(vf**2 + af**2)*&
	 (&
	 b0f(p1sq,mf,mf)+b0f(p2sq,mf,mf)+2.d0*b0f(qsq,mf,mf)&
	 +(4.d0*mf**2-p1sq-p2sq)*c0f(p1sq,p2sq,qsq,mf,mf,mf)&
	 -8.d0*c24f(p1sq,p2sq,qsq,mf,mf,mf)&
	 )&
	 +1.d0/2.d0*(vf**2-af**2)*&
	   (&
	 +b0f(p2sq,mf,mf)+b0f(p1sq,mf,mf)&
	 +(4.d0*mf**2-qsq)*c0f(p1sq,p2sq,qsq,mf,mf,mf)&
	   )&
	 )

 	 end function
!------------------------------------------------------------------
 	complex (kind(0d0)) function Gam2_hZZ_fermion(p1sq,p2sq,qsq,mf)
 	double precision, intent(in):: p1sq,p2sq,qsq,mf
 	double precision:: vf,af

       if (mf .eq. mt) then 
        vf = vt ; af = at
       elseif (mf .eq. mc) then 
        vf = vt ; af = at
       elseif (mf .eq. mb) then 
        vf = vb ; af = ab
       elseif (mf .eq. mtau) then 
        vf = ve ; af = ae
       endif

	Gam2_hZZ_fermion = &
	 -32.d0*mf**2*mz**4/vev**3/(16.d0*pi**2)*&
	 (&
	 (vf**2 + af**2)*&
	 (4.d0*c23f(p1sq,p2sq,qsq,mf,mf,mf)&
          +3.d0*c12f(p1sq,p2sq,qsq,mf,mf,mf)&
          +c11f(p1sq,p2sq,qsq,mf,mf,mf)&
          +c0f(p1sq,p2sq,qsq,mf,mf,mf)&
          )&
         +(vf**2-af**2)*&
         (c12f(p1sq,p2sq,qsq,mf,mf,mf)&
          -c11f(p1sq,p2sq,qsq,mf,mf,mf)&
          )&
	 )

 	 end function
!----------------------------------------------------------------
 	complex (kind(0d0)) function Gam3_hZZ_fermion(p1sq,p2sq,qsq,mf)
 	double precision, intent(in):: p1sq,p2sq,qsq,mf
 	double precision:: vf,af

       if (mf .eq. mt) then 
        vf = vt ; af = at
       elseif (mf .eq. mc) then 
        vf = vt ; af = at
       elseif (mf .eq. mb) then 
        vf = vb ; af = ab
       elseif (mf .eq. mtau) then 
        vf = ve ; af = ae
       endif

	Gam3_hZZ_fermion = 64.d0*mf**2*mz**4/vev**3/(16.d0*pi**2)*af*vf*&
         (c11f(p1sq,p2sq,qsq,mf,mf,mf)+c12f(p1sq,p2sq,qsq,mf,mf,mf)+c0f(p1sq,p2sq,qsq,mf,mf,mf)&
         )

 	 end function
!-------------------------------------------------------------------
 	complex (kind(0d0)) function Gam1_hWW_fermion(p1sq,p2sq,qsq,mf)
 	double precision, intent(in):: p1sq,p2sq,qsq,mf
 	double precision:: mfp

       if (mf .eq. mt) mfp = mb
       if (mf .eq. mb) mfp = mt
       if (mf .eq. mc) mfp = ms
       if (mf .eq. ms) mfp = mc
       if (mf .eq. mtau) mfp = 0.d0

 	 Gam1_hWW_fermion =4.d0*mw**2*mf**2/vev**3/&
 	 (16.d0*pi**2)*&
 	 (&
 	 1.d0/2.d0*b0f(p2sq,mf,mfp)+b0f(qsq,mf,mf)&
 	 +1.d0/2.d0*b0f(p1sq,mf,mfp)&
 	 -4.d0*c24f(p1sq,p2sq,qsq,mf,mfp,mf)&
 	 +(mf**2+mfp**2-p1sq/2.d0-p2sq/2.d0)*c0f(p1sq,p2sq,qsq,mf,mfp,mf)&
 	 )

 	 end function
!--------------------------------------------
 	complex (kind(0d0)) function Gam2_hWW_fermion(p1sq,p2sq,qsq,mf)
 	double precision, intent(in):: p1sq,p2sq,qsq,mf
 	double precision:: mfp

       if (mf .eq. mt) mfp = mb
       if (mf .eq. mb) mfp = mt
       if (mf .eq. mc) mfp = ms
       if (mf .eq. ms) mfp = mc
       if (mf .eq. mtau) mfp = 0.d0

	Gam2_hWW_fermion =&
	 -4.d0*mw**4*mf**2/vev**3/(16.d0*pi**2)*&
	 (4.d0*c23f(p1sq,p2sq,qsq,mf,mfp,mf)&
         +3.d0*c12f(p1sq,p2sq,qsq,mf,mfp,mf)&
         +c11f(p1sq,p2sq,qsq,mf,mfp,mf)&
         +c0f(p1sq,p2sq,qsq,mf,mfp,mf)&
	 )

	 end function
!--------------------------------------------
 	complex (kind(0d0)) function Gam3_hWW_fermion(p1sq,p2sq,qsq,mf)
 	double precision, intent(in):: p1sq,p2sq,qsq,mf
 	double precision:: mfp

       if (mf .eq. mt) mfp = mb
       if (mf .eq. mb) mfp = mt
       if (mf .eq. mc) mfp = ms
       if (mf .eq. ms) mfp = mc
       if (mf .eq. mtau) mfp = 0.d0

	Gam3_hWW_fermion =&
	 -4.d0*mw**4*mf**2/vev**3/(16.d0*pi**2)*&
	 (c11f(p1sq,p2sq,qsq,mf,mfp,mf)&
         +c12f(p1sq,p2sq,qsq,mf,mfp,mf)&
         +c0f(p1sq,p2sq,qsq,mf,mfp,mf)&
	 )

	 end function
!=================================================================
	complex (kind(0d0)) function Gam1_hZZ_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam1_hZZ_F_SM = nc*Gam1_hZZ_fermion(p1sq,p2sq,qsq,mt)&
                   +nc*Gam1_hZZ_fermion(p1sq,p2sq,qsq,mc)&
                   +nc*Gam1_hZZ_fermion(p1sq,p2sq,qsq,mb)&
                   +Gam1_hZZ_fermion(p1sq,p2sq,qsq,mtau)
	
	 end function

!--------------------------------------------
	complex (kind(0d0)) function Gam2_hZZ_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam2_hZZ_F_SM =  nc*Gam2_hZZ_fermion(p1sq,p2sq,qsq,mt)&
                   +nc*Gam2_hZZ_fermion(p1sq,p2sq,qsq,mc)&
                   +nc*Gam2_hZZ_fermion(p1sq,p2sq,qsq,mb)&
                   +Gam2_hZZ_fermion(p1sq,p2sq,qsq,mtau)
	
	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam3_hZZ_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam3_hZZ_F_SM = nc*Gam3_hZZ_fermion(p1sq,p2sq,qsq,mt)&
                   +nc*Gam3_hZZ_fermion(p1sq,p2sq,qsq,mc)&
                   +nc*Gam3_hZZ_fermion(p1sq,p2sq,qsq,mb)&
                   +Gam3_hZZ_fermion(p1sq,p2sq,qsq,mtau)

	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam1_hZZ_B_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	double precision:: p12
	p12=(qsq-p1sq-p2sq)/2.d0

	Gam1_hZZ_B_SM = &
	 -gz**2*1.d0/(16.d0*pi**2)*&
	 (&
	 +2.d0*lam_GpGmh_sm(mh)*(cw**2-sw**2)**2*c24f(p1sq,p2sq,qsq,mw,mw,mw)&
	 +2.d0*lam_GGh_sm(mh)*c24f(p1sq,p2sq,qsq,mz,mh,mz)&
	 +6.d0*lam_hhh_sm(mh)*c24f(p1sq,p2sq,qsq,mh,mz,mh)&
	 )&

	 +gz**2*1.d0/(16.d0*pi**2)*&
	 (&
	 +1.d0/2.d0*lam_GpGmh_sm(mh)*(cw**2-sw**2)**2*b0f(qsq,mw,mw)&
	 +1.d0/2.d0*lam_GGh_sm(mh)*b0f(qsq,mz,mz)&
	 +3.d0/2.d0*lam_hhh_sm(mh)*b0f(qsq,mh,mh)&
	 )&
!~~~~~~~~~~~ Gauge Boson Loops ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
	+1.d0/(16.d0*pi**2)*(&
	 +2.d0*g**3*mw*cw**2*ChVV1_VVV(p1sq,p2sq,qsq,mW,mW,mW)&
	 +g**3*mw*sw**2*ChVV1_SVV(p1sq,p2sq,qsq,mw,mW,mW)& 
	 +g**3*mw*sw**2*ChVV1_VVS(p1sq,p2sq,qsq,mW,mW,mw)&	 
	 -2.d0*g**3*mw**3/cw**2*sw**4*c0f(p1sq,p2sq,qsq,mW,mw,mW)&
	 -2.d0*g**3*cw**2*mw*c24f(p1sq,p2sq,qsq,mw,mw,mw)&
	 -g*gz**2*mw*(cw**2-sw**2)*sw**2*&
	 ( c24f(p1sq,p2sq,qsq,mW,mw,mw)+c24f(p1sq,p2sq,qsq,mw,mw,mW) )&
	 +2.d0*gz**2*lam_GpGmh_sm(mh)*mw**2*sw**4*c0f(p1sq,p2sq,qsq,mw,mW,mw)&
	 -6.d0*g**3*cw**2*mw*b0f(qsq,mW,mW)+4.d0*g**3*cw**2*mw&
	 -g*gz**2*mw*sw**4*( b0f(p2sq,mW,mw)+b0f(p1sq,mw,mW) )&
!
	 +gz**3/2.d0*mz*( c24f(p1sq,p2sq,qsq,mZ,mh,mz)+c24f(p1sq,p2sq,qsq,mz,mh,mZ) )&
	 -gz**3*mz**3*c0f(p1sq,p2sq,qsq,mZ,mh,mZ)&
	 +6.d0*lam_hhh_sm(mh)*gz**2*mz**2*c0f(p1sq,p2sq,qsq,mh,mZ,mh)&
	 -1.d0/2.d0*gz**3*mz*( b0f(p1sq,mh,mZ)+b0f(p2sq,mh,mZ))&
	 )

	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam2_hZZ_B_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	double precision:: p12
	p12=(qsq-p1sq-p2sq)/2.d0

	Gam2_hZZ_B_SM = gz**2/(16.d0*pi**2)*mz**2*(&
                   +2.d0*g*mw*cw**4*(ChVV2_VVV(p1sq,p2sq,qsq,mW,mW,mW) - C1223(p1sq,p2sq,qsq,mw,mw,mw)) &
!
                   +g*mw*sw**2*cw**2*ChVV2_SVV(p1sq,p2sq,qsq,mw,mW,mW)&
                   +g*mw*sw**2*cw**2*ChVV2_VVS(p1sq,p2sq,qsq,mW,mW,mw)&
!
                   -g*mw*(cw**2-sw**2)*sw**2*ChVV2_VSS(p1sq,p2sq,qsq,mW,mw,mw)&
                   -g*mw*(cw**2-sw**2)*sw**2*ChVV2_SSV(p1sq,p2sq,qsq,mw,mw,mW)&
                   +gz/2.d0*mz*ChVV2_VSS(p1sq,p2sq,qsq,mZ,mh,mz)&
                   +gz/2.d0*mz*ChVV2_SSV(p1sq,p2sq,qsq,mz,mh,mZ)&
! 
                   -2.d0*lam_GpGmh_sm(mh)*(cw**2-sw**2)**2*c1223(p1sq,p2sq,qsq,mw,mw,mw)&
                   -2.d0*lam_GGh_sm(mh)*c1223(p1sq,p2sq,qsq,mz,mh,mz)&
                   -6.d0*lam_hhh_sm(mh)*c1223(p1sq,p2sq,qsq,mh,mz,mh)&
                    )

	 end function
!==============================================================================
	complex (kind(0d0)) function Gam1_hWW_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam1_hWW_F_SM = nc*Gam1_hWW_fermion(p1sq,p2sq,qsq,mt)&
                   +nc*Gam1_hWW_fermion(p1sq,p2sq,qsq,mb)&
                   +nc*Gam1_hWW_fermion(p1sq,p2sq,qsq,mc)&
                   +nc*Gam1_hWW_fermion(p1sq,p2sq,qsq,ms)&
                   +Gam1_hWW_fermion(p1sq,p2sq,qsq,mtau)

	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam2_hWW_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam2_hWW_F_SM = nc*Gam2_hWW_fermion(p1sq,p2sq,qsq,mt)&
                   +nc*Gam2_hWW_fermion(p1sq,p2sq,qsq,mb)&
                   +nc*Gam2_hWW_fermion(p1sq,p2sq,qsq,mc)&
                   +nc*Gam2_hWW_fermion(p1sq,p2sq,qsq,ms)&
                   +Gam2_hWW_fermion(p1sq,p2sq,qsq,mtau)

	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam3_hWW_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam3_hWW_F_SM = nc*Gam3_hWW_fermion(p1sq,p2sq,qsq,mt)&
                   +nc*Gam3_hWW_fermion(p1sq,p2sq,qsq,mb)&
                   +nc*Gam3_hWW_fermion(p1sq,p2sq,qsq,mc)&
                   +nc*Gam3_hWW_fermion(p1sq,p2sq,qsq,ms)&
                      +Gam3_hWW_fermion(p1sq,p2sq,qsq,mtau)

	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam1_hWW_B_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	double precision:: p12
	p12=(qsq-p1sq-p2sq)/2.d0

	Gam1_hWW_B_SM =&
	  g**2*1.d0/(16.d0*pi**2)*&
	 (&
	 -2.d0*lam_GGh_sm(mh)*c24f(p1sq,p2sq,qsq,mz,mw,mz)&
	 -6.d0*lam_hhh_sm(mh)*(c24f(p1sq,p2sq,qsq,mh,mw,mh) )&
	 -lam_GpGmh_sm(mh)*( c24f(p1sq,p2sq,qsq,mw,mz,mw)   )&
	 -lam_GpGmh_sm(mh)*( c24f(p1sq,p2sq,qsq,mw,mh,mw)   )&
	 )&
!
	 +g**2/(16.d0*pi**2)*&
	 (&
	 +1.d0/2.d0*lam_GpGmh_sm(mh)*b0f(qsq,mw,mw)&
	 +1.d0/2.d0*lam_GGh_sm(mh)*b0f(qsq,mz,mz)&
	 +3.d0/2.d0*lam_hhh_sm(mh)*b0f(qsq,mh,mh)&
	 )&
!~~~~~~~~~~~ Gauge Boson Loops ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
	  +1.d0/(16.d0*pi**2)*&
	 (&
	 +g**3*cw*mz*ChVV1_VVV(p1sq,p2sq,qsq,mZ,mW,mZ)&
	 +g**3*cw**2*mw*ChVV1_VVV(p1sq,p2sq,qsq,mW,mZ,mW)&
	 +g**3*sw**2*mw*ChVV1_VVV(p1sq,p2sq,qsq,mW,mphoton,mW)&
	 -1.d0/2.d0*g**3*mw*sw**2*(ChVV1_SVV(p1sq,p2sq,qsq,mw,mZ,mW)-ChVV1_SVV(p1sq,p2sq,qsq,mw,mphoton,mW))& 
	 -1.d0/2.d0*g**3*mw*sw**2*(ChVV1_VVS(p1sq,p2sq,qsq,mW,mZ,mw)-ChVV1_VVS(p1sq,p2sq,qsq,mW,mphoton,mw))& 
	 -g**3/cw**3*mw**2*mZ*sw**4*c0f(p1sq,p2sq,qsq,mZ,mw,mZ)&
	 -g**3*mw**3*c0f(p1sq,p2sq,qsq,mW,mh,mW)&
	 +6.d0*g**2*lam_hhh_sm(mh)*mw**2*c0f(p1sq,p2sq,qsq,mh,mW,mh)& 
	 +lam_GpGmh_sm(mh)*g**2*mw**2*sw**4/cw**2*c0f(p1sq,p2sq,qsq,mw,mZ,mw)&
	 +lam_GpGmh_sm(mh)*g**2*sw**2*mw**2*c0f(p1sq,p2sq,qsq,mw,mphoton,mw)&
	 +1.d0/2.d0*g**3*mw*( c24f(p1sq,p2sq,qsq,mW,mh,mw) + c24f(p1sq,p2sq,qsq,mw,mh,mW) )&
	 +1.d0/2.d0*g**3*sw**2/cw**2*mw*( c24f(p1sq,p2sq,qsq,mz,mw,mZ) + c24f(p1sq,p2sq,qsq,mZ,mw,mz) )&
!
	 -g**3*cw*mz*c24f(p1sq,p2sq,qsq,mz,mw,mz)&
	 -g**3*cw**2*mw*c24f(p1sq,p2sq,qsq,mw,mz,mw)&
	 -g**3*sw**2*mw*c24f(p1sq,p2sq,qsq,mw,mphoton,mw)&
	 -3.d0*g**3*mw*b0f(qsq,mW,mW)+2.d0*g**3*mw&
	 -3.d0*g**3*cw*mz*b0f(qsq,mZ,mZ)+2.d0*g**3*cw*mz&
!
	 -1.d0/2.d0*g**3*mw*( b0f(p1sq,mW,mh) + b0f(p2sq,mW,mh) )&
	 -1.d0/2.d0*g**3*mw*sw**4/cw**2*( b0f(p1sq,mZ,mw) + b0f(p2sq,mZ,mw) )&
	 -1.d0/2.d0*g**3*sw**2*mw*( b0f(p1sq,mphoton,mw) + b0f(p2sq,mphoton,mw) )&
	 )     

	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam2_hWW_B_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	double precision:: p12
	p12=(qsq-p1sq-p2sq)/2.d0

	Gam2_hWW_B_SM = g**2/(16.d0*pi**2)*mw**2*(&
                   +g*cw*mz*(ChVV2_VVV(p1sq,p2sq,qsq,mZ,mW,mZ) - c1223(p1sq,p2sq,qsq,mz,mw,mz)) &
                   +g*mw*cw**2*(ChVV2_VVV(p1sq,p2sq,qsq,mW,mZ,mW)-c1223(p1sq,p2sq,qsq,mw,mz,mw))& 
                   +g*mw*sw**2*(ChVV2_VVV(p1sq,p2sq,qsq,mW,mphoton,mW)-c1223(p1sq,p2sq,qsq,mw,mphoton,mw)) &
!
                    -g/2.d0*mw*(sw**2*ChVV2_SVV(p1sq,p2sq,qsq,mw,mZ,mW) - sw**2*ChVV2_SVV(p1sq,p2sq,qsq,mw,mphoton,mW))&
                    -g/2.d0*mw*(sw**2*ChVV2_VVS(p1sq,p2sq,qsq,mW,mZ,mw) - sw**2*ChVV2_VVS(p1sq,p2sq,qsq,mW,mphoton,mw))&
!
                    +g/2.d0*mw*ChVV2_VSS(p1sq,p2sq,qsq,mW,mh,mw)&
                    +g/2.d0*mw*ChVV2_SSV(p1sq,p2sq,qsq,mw,mh,mW)&
                    +g/2.d0*mw*sw**2/cw**2*ChVV2_VSS(p1sq,p2sq,qsq,mZ,mw,mz)&
                    +g/2.d0*mw*sw**2/cw**2*ChVV2_SSV(p1sq,p2sq,qsq,mz,mw,mZ)&
! 
                    -2.d0*lam_GGh_sm(mh)*c1223(p1sq,p2sq,qsq,mz,mw,mz)&
                    -1.d0*lam_GpGmh_sm(mh)*c1223(p1sq,p2sq,qsq,mw,mh,mw)&
                    -6.d0*lam_hhh_sm(mh)*c1223(p1sq,p2sq,qsq,mh,mw,mh)&
                    -1.d0*lam_GpGmh_sm(mh)*c1223(p1sq,p2sq,qsq,mw,mz,mw)&
                   )

	 end function
!##########################################################
!
!    SECTION III COUNTER-TERMS
!
!##########################################################
	double precision function del_mf(mf)
	double precision, intent(in):: mf
	double precision:: vf,af,qf,mfp
        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	del_mf = mf*(Piff_V_SM(mf**2,mf)+Piff_S_SM(mf**2,mf) )

	 end function
!---------------------------------------------
	double precision function del_ZVf(mf)
	double precision, intent(in):: mf
	double precision:: vf,af,qf,mfp
        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	del_ZVf = -Piff_V_SM(mf**2,mf)-2.d0*mf**2*(dPiff_V_SM(mf**2,mf)+dPiff_S_SM(mf**2,mf))

	 end function
!---------------------------------------------
	double precision function del_swsq_SM()

	del_swsq_SM = cw**2*(PiZZ_F(mz**2)/mz**2-PiWW_F(mw**2)/mw**2)&
                     +cw**2*(PiZZ_B_SM(mz**2)/mz**2-PiWW_B_SM(mw**2)/mw**2) 

	 end function
!---------------------------------------------
	double precision function del_v_SM()

	del_v_SM = &
	 vev/2.d0*(&
	 PiWW_F(mw**2)/mw**2&
	 -delta_alpha &
	 -dPiGG_F(mz**2))&
!
	 +vev/2.d0*(&
	 PiWW_B_SM(mw**2)/mw**2&
	 +del_swsq_SM()/sw**2&
	 -dPiGG_B_SM(mz**2))

	 end function
!---------------------------------------------
  	double precision function del_mZsq_SM()
	
	del_mZsq_SM = PiZZ_F(mz**2)+PiZZ_B_SM(mz**2)

	 end function
!--------------------------------------------
 	double precision function del_mWsq_SM()

	del_mWsq_SM = PiWW_F(mw**2) + PiWW_B_SM(mw**2)

	 end function
!--------------------------------------------
  	double precision function del_ZZ_SM()

	del_ZZ_SM =& 
	 -delta_alpha-dPiGG_F(mz**2)&
	   -2.d0*(cw**2-sw**2)/(cw*sw)*PiZG_F(0.d0)/mz**2&
            -(dPiGG_B_SM(mz**2)-dPiGG_pinch(mz**2))&
	   +2.d0*(cw**2-sw**2)/(cw*sw)*( PiZG_B_SM(0.d0)-PiZG_pinch(0.d0) )/mz**2&
	 +(cw**2-sw**2)/cw**2*del_swsq_SM()/sw**2

	 end function
!--------------------------------------------
  	double precision function del_ZW_SM()

	del_ZW_SM =&
	 -delta_alpha-dPiGG_F(mz**2)&
	   +2.d0*cw/sw*&
	 PiZG_F(0.d0)/mz**2&
	   -(dPiGG_B_SM(mz**2) -dPiGG_pinch(mz**2))&
	   +2.d0*cw/sw*(PiZG_B_SM(0.d0)-PiZG_pinch(0.d0))/mz**2&
	 +del_swsq_SM()/sw**2

	 end function
!--------------------------------------------
 	double precision function del_Th_SM()

 	del_Th_SM = -Th_F_SM()-Th_B_SM()

 	 end function
!---------------------------------------------
	double precision function del_mhsq_SM()
	
        del_mhsq_SM = del_Th_SM()/vev + Pihh_F_SM(mh**2)+Pihh_B_SM(mh**2)

        end function
!--------------------------------------------
	double precision function del_Zh_SM()

	del_Zh_SM = -dPihh_F_SM(mh**2) -dPihh_B_SM(mh**2)

	 end function
!--------------------------------------------
  	double precision function del_hhh_SM()

	del_hhh_SM = &
	 -6.d0*(&
	   (1.d0/vev)*del_mhsq_SM()/2.d0&
	 +3.d0/4.d0*(mh**2/vev)*del_Zh_SM()&
	 +mh**2/(2.d0*vev**3)*(-vev*del_v_SM())&
	)

	 end function
!--------------------------------------------
  	double precision function del_Gam1_hZZ_SM()

	del_Gam1_hZZ_SM = 2.d0*mz**2/vev*&
	 (&
	 +del_mzsq_SM()/mz**2&
	 +1.d0/2.d0*(del_Zh_SM()&
         +2.d0*del_ZZ_SM() )&
	 -del_v_SM()/vev&
	 )

	 end function
!--------------------------------------------
  	double precision function del_Gam1_hWW_SM()

	del_Gam1_hWW_SM = 2.d0*mw**2/vev*(&
	 +del_mWsq_SM()/mw**2&
	 +del_ZW_SM()&
	 +1.d0/2.d0*del_Zh_SM()&
	 -del_v_SM()/vev&
	 )

	 end function
!##########################################################
!
!    SECTION V RENORMALIZED HIGGS COUPLINGS     
!
!##########################################################
  	complex (kind(0d0)) function Gam_hhh_loop_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam_hhh_loop_SM =&
	 +Gam_hhh_F_SM(p1sq,p2sq,qsq)&
	 +Gam_hhh_B_SM(p1sq,p2sq,qsq)&
	 +del_hhh_SM()

	 end function
!---------------------------------------------
	complex (kind(0d0)) function Gam_hbb_loop_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam_hbb_loop_SM =&
	+Gam_hff_S_SM(p1sq,p2sq,qsq,mb)&
	-mb/vev*(&
	del_mf(mb)/mb-del_v_SM()/vev&
	+del_ZVf(mb)+1.d0/2.d0*del_Zh_SM()& 
	)

	end function
!---------------------------------------------
	complex (kind(0d0)) function Gam_htt_loop_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam_htt_loop_SM =&
	+Gam_hff_S_SM(p1sq,p2sq,qsq,mt)&
	-mt/vev*(&
	del_mf(mt)/mt-del_v_SM()/vev&
	+del_ZVf(mt)+1.d0/2.d0*del_Zh_SM()& 
	)

	end function
!---------------------------------------------
	complex (kind(0d0)) function Gam_hcc_loop_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam_hcc_loop_SM =&
	+Gam_hff_S_SM(p1sq,p2sq,qsq,mc)&
	-mc/vev*(&
	del_mf(mc)/mc-del_v_SM()/vev&
	+del_ZVf(mc)+1.d0/2.d0*del_Zh_SM()& 
	)

	end function
!---------------------------------------------
	complex (kind(0d0)) function Gam_htautau_loop_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam_htautau_loop_SM =&
	+Gam_hff_S_SM(p1sq,p2sq,qsq,mtau)&
	-mtau/vev*(&
	del_mf(mtau)/mtau-del_v_SM()/vev&
	+del_ZVf(mtau)+1.d0/2.d0*del_Zh_SM()& 
	)

	end function
!--------------------------------------------
  	complex (kind(0d0)) function Gam1_hZZ_loop_SM(p1sq,p2sq,qsq)
  	double precision, intent(in):: p1sq,p2sq,qsq
	
	Gam1_hZZ_loop_SM = &
	 +Gam1_hZZ_F_SM(p1sq,p2sq,qsq)&
	 +Gam1_hZZ_B_SM(p1sq,p2sq,qsq)&
      	 +del_Gam1_hZZ_SM()

	 end function
!--------------------------------------------
  	complex (kind(0d0)) function Gam1_hWW_loop_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	
	Gam1_hWW_loop_SM = &
	 +Gam1_hWW_F_SM(p1sq,p2sq,qsq)&
	 +Gam1_hWW_B_SM(p1sq,p2sq,qsq)&
	 +del_Gam1_hWW_SM()

	 end function
!==============================================================
  	complex (kind(0d0)) function rGam1_hZZ_SM(p1sq,p2sq,qsq)
  	double precision, intent(in):: p1sq,p2sq,qsq
	
	 rGam1_hZZ_SM = 2.d0*mZ**2/vev + Gam1_hZZ_loop_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
  	complex (kind(0d0)) function rGam2_hZZ_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

        rGam2_hZZ_SM =  Gam2_hZZ_F_SM(p1sq,p2sq,qsq) + Gam2_hZZ_B_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
   	complex (kind(0d0)) function rGam3_hZZ_SM(p1sq,p2sq,qsq)
 	double precision, intent(in):: p1sq,p2sq,qsq

         rGam3_hZZ_SM =  Gam3_hZZ_F_SM(p1sq,p2sq,qsq) 

         end function
!--------------------------------------------
  	complex (kind(0d0)) function rGam1_hWW_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	
	rGam1_hWW_SM = 2.d0*mW**2/vev + Gam1_hWW_loop_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
  	complex (kind(0d0)) function rGam2_hWW_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

        rGam2_hWW_SM =  Gam2_hWW_F_SM(p1sq,p2sq,qsq) + Gam2_hWW_B_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
   	complex (kind(0d0)) function rGam3_hWW_SM(p1sq,p2sq,qsq)
 	double precision, intent(in):: p1sq,p2sq,qsq

         rGam3_hWW_SM =  Gam3_hWW_F_SM(p1sq,p2sq,qsq) 

         end function
!--------------------------------------------
  	complex (kind(0d0)) function rGamS_hbb_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

        rGamS_hbb_SM = -mb/vev + Gam_hbb_loop_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
  	complex (kind(0d0)) function rGamS_htt_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

        rGamS_htt_SM = -mt/vev + Gam_htt_loop_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
  	complex (kind(0d0)) function rGamS_hcc_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

        rGamS_hcc_SM = -mc/vev + Gam_hcc_loop_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
  	complex (kind(0d0)) function rGamS_htautau_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

         rGamS_htautau_SM = -mtau/vev + Gam_htautau_loop_SM(p1sq,p2sq,qsq)
 
	 end function
!--------------------------------------------
  	complex (kind(0d0)) function rGam_hhh_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

        rGam_hhh_SM = 6.d0*lam_hhh_SM(mh) + Gam_hhh_loop_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
	end module HCOUP_SM
